unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Menus, Grids,
  Base, PuzzleIO;

type
  TForm1 = class(TForm)
    BoxGrid: TDrawGrid;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Exit1: TMenuItem;
    N1: TMenuItem;
    PrintSetup1: TMenuItem;
    Print1: TMenuItem;
    N2: TMenuItem;
    SaveAs1: TMenuItem;
    Save1: TMenuItem;
    Open1: TMenuItem;
    New1: TMenuItem;
    Label1: TLabel;
    Label2: TLabel;
    OpenDialog1: TOpenDialog;
    SaveDialog1: TSaveDialog;
    PiecesGrid: TStringGrid;
    procedure FormCreate(Sender: TObject);
    procedure Open1Click(Sender: TObject);
    procedure BoxGridClick(Sender: TObject);
    procedure BoxGridDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure PiecesGridDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
  private
    { Private declarations }
  public
    { Public declarations }
    FPuzzle: TPuzzle;
    procedure DoReadPuzzle(AFileName: String);
    procedure DoPlacePieces;
    procedure UpdatePiecesGrid;
    procedure UpdateBoxGrid;
    procedure UpdateViews;
  end;

const
  fnDefaultPuzzleFileName = 'MyPuzzle.txt';

var
  Form1: TForm1;

implementation //===============================================================

uses
  Math;

{$R *.dfm}

{ TForm1 }

procedure TForm1.DoPlacePieces;
var
  I: Integer; // to traverse the pieces
  J: Integer; // to traverse the position covered by a placement
  VPos: TPosition; // current position in PiecesGrid
  VPiece: TPiece; // piece being placed
  VOrientation: TOrientation; // first orientation of VPiece
  VPieceSize: Integer; // max width of VPiece orientation
  VPlacement: TPlacement; // of VPiece
  VPositionList: TPositionList; // to capture result of CoverP
  VRowCount, VColCount: Integer; // to determine size of PiecesGrid
begin
  VColCount := 1;
  VRowCount := 1;
  VPos.Col := 1;
  VPos.Row := 2;

  // calculate placements for pieces in PiecesGrid, depending on their max size
  with FPuzzle.Stock do
  begin
    for I := 0 to Count - 1 do
    begin
      VPiece := GetPiece(I);
      VOrientation := VPiece.GetOrientation(0);
      VPlacement := TPlacement.Create(VPiece, VOrientation, VPos);

      // determine size, update dimensions of PiecesGrid
      VPieceSize := Max(VOrientation.MaxCol + 1, VOrientation.MaxRow + 1);
      VPos.Col := VPos.Col + VPieceSize + 1;
      VRowCount := Max(VRowCount, VPieceSize + 2);
      VColCount := VPos.Col + 1;

      // set dimensions for Piecesgrid
      PiecesGrid.ColCount := VColCount;
      PiecesGrid.RowCount := VRowCount;

      // Assign placement to corresponding cells in PiecesGrid,
      // write name on top
      VPositionList := CoverP(VPlacement);

      for J := 0 to Length(VPositionList) - 1 do
        with VPositionList[J] do
        begin
          PiecesGrid.Objects[Col, Row] := VPlacement;
        end; { with }

      with VPositionList[0] do
        PiecesGrid.Cells[Col, 0] := VPlacement.FPiece.Name;
    end; { for I }

  end; { with }
end;


procedure TForm1.DoReadPuzzle(AFileName: String);
begin
  FPuzzle := ReadPuzzle(AFileName);  // memory leak!
end;


procedure TForm1.UpdateBoxGrid;
begin
  BoxGrid.RowCount := FPuzzle.Box.RowCount;
  BoxGrid.ColCount := FPuzzle.Box.ColCount;
end;

procedure TForm1.UpdatePiecesGrid;
begin

end;

procedure TForm1.UpdateViews;
begin
  UpdatePiecesGrid;
  UpdateBoxGrid;
end;

procedure TForm1.FormCreate(Sender: TObject);
var
  VPiece: TPiece;
  VPosition: TPosition;
  VPlacement: TPlacement;
begin
  DoReadPuzzle(fnDefaultPuzzleFileName);
  DoPlacePieces;

// Temporarily, for testing purposes -------------------------------------------
    VPiece := FPuzzle.Stock.FindPiece('A');
    VPosition.Col := 1;
    VPosition.Row := 0;
    VPlacement := TPlacement.Create(VPiece, VPiece.GetOrientation(0), VPosition);
    FPuzzle.DoPlacement(VPlacement);

    VPiece := FPuzzle.Stock.FindPiece('B');
    VPosition.Col := 0;
    VPosition.Row := 0;
    VPlacement := TPlacement.Create(VPiece, VPiece.GetOrientation(1), VPosition);
    FPuzzle.DoPlacement(VPlacement);

//------------------------------------------------------------------------------

  UpdateViews;
end;

procedure TForm1.Open1Click(Sender: TObject);
begin
  if OpenDialog1.Execute then
  DoReadPuzzle(OpenDialog1.FileName);
  DoPlacePieces;
  UpdateViews;
end;

procedure TForm1.BoxGridClick(Sender: TObject);
begin
  UpdateViews;
  with BoxGrid do
  begin
    Canvas.Brush.Color := clRed;
    Canvas.FillRect(CellRect(Col,Row));
  end;
end;

procedure TForm1.BoxGridDrawCell(Sender: TObject; ACol, ARow: Integer;
            Rect: TRect; State: TGridDrawState);
var
  VColor: TColor;
  VState: TCellState;
begin
  with FPuzzle.Box do
  begin
    if ValidCoordinates(ARow, ACol) then
    begin
      VState := FPuzzle.Box.CellState(ARow, ACol);
      case VState of
        csFree:
          begin
            VColor := clWhite;
          end;
        csOccupied:
          begin
            VColor := FPuzzle.Box.CellOccupant(ARow, ACol).FPiece.Color;
          end;
        csBlocked:
          begin
            VColor := clBlack;
          end;
      end;{case}
     end;{if}
    end;{with}

  BoxGrid.Canvas.Brush.Color := VColor;
  BoxGrid.Canvas.FillRect(Rect);
end;

procedure TForm1.PiecesGridDrawCell(Sender: TObject; ACol, ARow: Integer;
            Rect: TRect; State: TGridDrawState);
var
  VPiece: TPiece;
  VColor: TColor;
begin
  with PiecesGrid do
    if Cells[ACol, ARow] = '' then
    // tricky, assumes that text cells and piece cells are disjoint
    begin
      // determine color
      if Objects[ACol, ARow] = nil
      then VColor := clWhite
      else
        begin
          VPiece := (Objects[ACol, ARow] as TPLacement).FPiece;
          with FPuzzle.Stock do
            if VPiece.Multiplicity > 0
            then VColor := VPiece.Color
            else VColor := clLtGray;
        end;

      // draw cell
      Canvas.Brush.Color := VColor;
      Canvas.FillRect(Rect);
    end;{with}
end;

end.
